{******************************************************************************}
{                                                                              }
{           ReportBuilder Data Access Developement Environment (DADE)          }
{                                                                              }
{             Copyright (c) 2000 Carval Computing Limited                      }
{             Copyright (c) 1996-1999 Digital Metaphors Corporation            }
{                                                                              }
{                                   v5.5                                       }
{******************************************************************************}

{******************************************************************************}
{                                                                              }
{                       Titan SQLAnywhere for Delphi 5                         }
{                                                                              }
{******************************************************************************}

unit daDBTitanSQLA;

interface

uses Classes, SysUtils, Forms, ExtCtrls, ppClass, ppDBPipe, ppDB, ppDBBDE,
     ppClasUt, ppUtils, ppTypes, daDB, daQueryDataView, daDataView, daPreviewDataDlg,
     daSQL, SqaMain, BSEMain, DB;

type

  {Titan Dependent DataView Classes:

     1.  TitanSQLA TDataSet descendants
           - TDataSets that can be children of a DataView.
           - Override the HasParent method of TComponent to return True
           - Must be registerd with the Delphi IDE using the RegisterNoIcon procedure

       a. TdaChildTitanSQLAQuery - TQuery descendant that can be a child of a DataView
       b. TdaChildTitanSQLATable - TTable descendant that can be a child of a DataView
       c. TdaChildTitanSQLAStoredProc - TStoredProc descendant that can be a child of a DataView


     2.  TdaTitanSQLASession
           - descendant of TppSession
           - implements GetDatabaseNames, GetTableNames, etc.

     3.  TdaTitanSQLADataSet
          - descendant of TppDataSet
          - implements GetFieldNames for SQL

     4.  TdaTitanSQLAQueryDataView
          - descendant of TppQueryDataView
          - uses the above classes to create the required
            Query -> DataSource -> Pipeline -> Report connection
          - uses the SQL built by the QueryWizard to assign
            SQL to the TQuery etc.

      }

  { TdaChildTitanSQLAQuery }
  TdaChildTitanSQLAQuery = class(TtsQuery)
    public
      function HasParent: Boolean; override;
    end;  {class, TdaChildTitanSQLAQuery}

  { TdaChildTitanSQLATable }
  TdaChildTitanSQLATable = class(TtsTable)
    public
      function HasParent: Boolean; override;
    end;  {class, TdaChildTitanSQLATable}

  { TdaChildTitanSQLAStoredProc }
  TdaChildTitanStoredProc = class(TtsStoredProc)
    public
      function HasParent: Boolean; override;
    end;  {class, TdaChildTitanSQLAStoredProc}


  { TdaTitanSQLASession }
  TdaTitanSQLASession = class(TdaSession)
    private
      function GetAliasDriverName(const aAlias: String): String;
    protected
      procedure SetDataOwner(aDataOwner: TComponent); override;

    public
      class function ClassDescription: String; override;
      class function DataSetClass: TdaDataSetClass; override;
      class function DatabaseClass: TComponentClass; override;

      procedure GetDatabaseNames(aList: TStrings); override;
      function GetDatabaseType(const aDatabaseName: String): TppDatabaseType; override;
      procedure GetTableNames(const aDatabaseName: String; aList: TStrings); override;
      function ValidDatabaseTypes: TppDatabaseTypes; override;

  end; {class, TdaTitanSQLASession}


  { TdaTitanSQLADataSet }
  TdaTitanSQLADataSet = class(TdaDataSet)
    private
      FQuery: TtsQuery;

      function GetQuery: TtsQuery;

    protected
      procedure BuildFieldList; override;
      function  GetActive: Boolean; override;
      procedure SetActive(Value: Boolean); override;
      procedure SetDatabaseName(const aDatabaseName: String); override;
      procedure SetDataName(const aDataName: String); override;

      property Query: TtsQuery read GetQuery;

    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;

      class function ClassDescription: String; override;

      procedure GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings); override;
      procedure GetFieldsForSQL(aList: TList; aSQL: TStrings); override;

  end; {class, TdaTitanSQLADataSet}


  { TdaTitanSQLAQueryDataView }
  TdaTitanSQLAQueryDataView = class(TdaQueryDataView)
    private
      FDataSource: TppChildDataSource;
      FQuery: TdaChildTitanSQLAQuery;

    protected
      procedure Loaded; override;
      procedure UpdateQueryObject;

    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;

      class function PreviewFormClass: TFormClass; override;
      class function SessionClass: TClass; override;

      procedure Init; override;
      procedure ConnectPipelinesToData; override;

      procedure SQLChanged; override;

    published
      property DataSource: TppChildDataSource read FDataSource;

  end; {class, TdaTitanSQLAQueryDataView}




  { TdaTitanSQLATemplateDataView }
  TdaTitanSQLATemplateDataView = class(TdaCustomDataView)
    private
      FDatabaseName: String;

    protected
      procedure SetDatabaseName(aDatabaseName: String); virtual;

    public
      {defines TitanSQLASession as the session class}
      class function SessionClass: TClass; override;

      {returns True}
      class function IsTemplate: Boolean; override;

      {descendants should call these to create data access objects}
      function CreateQuery: TtsQuery;
      function CreateTable(aTableName: String): TtsTable;
      function CreateStoredProc(aStoredProcName: String): TtsStoredProc;
      function CreateDataSource: TDataSource;
      function CreateDataPipeline: TppDBPipeline;
      function CreatePipelineField(aTableName, aFieldName, aFieldAlias: String;
                             aDataPipeline: TppDataPipeline; aSearchable, aSortable: Boolean): TppField;

      property DatabaseName: String read FDatabaseName write SetDatabaseName;

    published

      property Report;

  end; {class, TdaTitanSQLATemplateDataView}



  { TdaTitanSQLAQueryTemplateDataView }
  TdaTitanSQLAQueryTemplateDataView = class(TdaTitanSQLATemplateDataView)
    private
      FQuery: TtsQuery;
      FDataSource: TDataSource;
      FDataPipeline: TppDBPipeline;
      FSQL: TdaSQL;

    protected
      procedure Loaded; override;
      procedure SetDatabaseName(aDatabaseName: String); override;

    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;

      procedure CreateDataPipelines; override;
      procedure CreatePipelineFields(aDataPipeline: TppDataPipeline); override;

      procedure ConnectPipelinesToData; override;
      procedure DefineDataSelection; override;

      procedure Init; override;

      function  AddSelectTable(aTableName: String): TdaTable;
      function  AddSelectField(aTable: TdaTable; aFieldName, aFieldAlias: String; aSearchable, aSortable: Boolean): TdaField;
      procedure DefineSelectedFields; virtual;
      procedure DefineCalculatedFields; virtual;
      procedure DefineSelectionCriteria; virtual;
      procedure DefineSortOrder; virtual;

      procedure BuildSQL;

      property SQL: TdaSQL read FSQL;

    published
      property DataPipeline: TppDBPipeline read FDataPipeline write FDataPipeline;
      property Query: TtsQuery read FQuery;
      property DataSource: TDataSource read FDataSource;

  end; {class, TdaTitanSQLAQueryTemplateDataView}


procedure Register;

implementation

procedure Register;
begin

  {Titan SQL Anywhere DataAccess Components}
  RegisterNoIcon([TdaChildTitanSQLAQuery, TdaChildTitanSQLATable, TdaChildTitanSQLAStoredProc]);

  {Titan SQL Anywhere DataViews}
  RegisterNoIcon([TdaTitanSQLAQueryDataView]);


end;


{******************************************************************************
 *
 ** C H I L D   T I T A N  D A T A   A C C E S S   C O M P O N  E N T S
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaChildTitanSQLAQuery.HasParent }

function TdaChildTitanSQLAQuery.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}

{------------------------------------------------------------------------------}
{ TdaChildTitanSQLATable.HasParent }

function TdaChildTitanSQLATable.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}


{------------------------------------------------------------------------------}
{ TdaChildTitanSQLAStoredProc.HasParent }

function TdaChildTitanSQLAStoredProc.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}




{******************************************************************************
 *
 ** T I T A N   S Q L A   S E S S I O N
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaTitanSQLASession.ClassDescription }

class function TdaTitanSQLASession.ClassDescription: String;
begin
  Result := 'TitanSQLAnywhereSession';
end; {class function, ClassDescription}


{------------------------------------------------------------------------------}
{ TdaTitanSQLASession.DataSetClass }

class function TdaTitanSQLASession.DataSetClass: TdaDataSetClass;
begin
  Result := TdaTitanSQLADataSet;
end; {class function, DataSetClass}

{------------------------------------------------------------------------------}
{ TdaTitanSession.DatabaseClass }

class function TdaTitanSQLASession.DatabaseClass: TComponentClass;
begin
  Result := TttDatabase;
end; {class function, DatabaseClass}

{------------------------------------------------------------------------------}
{ TdaTitanSQLASession.GetAliasDriverName }

function TdaTitanSQLASession.GetAliasDriverName(const aAlias: String): String;
var
  lDatabase: TttDatabase;
begin

 { Result := 'STANDARD';
  Exit;}

  Result := '';

  {try to get driver name from the session}
  if tsSession.IsAlias(aAlias) then
    Result := tsSession.GetAliasDriverName(aAlias)
  else
    begin

      {try to get the TDatabase object}
      lDatabase := tsSession.FindDatabase(aAlias);

      {return the drivername using Database.AliasName or Database.DriverName as appropriate}
      if lDatabase <> nil then
        if taSession.IsAlias(lDatabase.AliasName) then
          Result := tsSession.GetAliasDriverName(lDatabase.AliasName)
        else
          Result := lDatabase.DatabaseName;

    end;

end; {function, GetAliasDriverName}


{------------------------------------------------------------------------------}
{ TdaTitanSQLASession.GetTableNames }

procedure TdaTitanSQLASession.GetTableNames(const aDatabaseName: String; aList: TStrings);
begin
  tsSession.GetTableNames(aDatabaseName, '*.*', True, False, aList);
end; {procedure, GetTableNames}

{------------------------------------------------------------------------------}
{ TdaTitanSQLASession.GetDatabaseNames }

procedure TdaTitanSQLASession.GetDatabaseNames(aList: TStrings);
begin
  tsSession.GetDataBaseNames(aList);
end; {procedure, GetDatabaseNames}

{------------------------------------------------------------------------------}
{ TdaTitanSQLASession.GetDatabaseType }
function TdaTitanSQLASession.GetDatabaseType(const aDatabaseName: String): TppDatabaseType;
Begin
     Result := dtSybaseASA;
End; {function, GetDatabaseType}

{------------------------------------------------------------------------------}
{ TdaTitanSQLASession.SetDataOwner }
procedure TdaTitanSQLASession.SetDataOwner(aDataOwner: TComponent);
Var
   lList: TStringList;
Begin
     inherited SetDataOwner(aDataOwner);

     lList := TStringList.Create;

     GetDatabaseNames(lList);

     lList.Free;
End; {procedure, SetDataOwner}

{------------------------------------------------------------------------------}
{ TdaTitanSQLASession.ValidDatabaseTypes }
function TdaTitanSQLASession.ValidDatabaseTypes: TppDatabaseTypes;
Begin
     Result := [dtSybaseASA];
End; {function, ValidDatabaseTypes}
{******************************************************************************
 *
 ** T I T A N   S Q L A   D A T A S E T
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaTitanSQLADataSet.Create }

constructor TdaTitanSQLADataSet.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  FQuery := nil;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaTitanSQLADataSet.Destroy }

destructor TdaTitanSQLADataSet.Destroy;
begin

  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdaTitanSQLADataSet.ClassDescription }

class function TdaTitanSQLADataSet.ClassDescription: String;
begin
  Result := 'TitanSQLAnywhereDataSet';
end; {class function, ClassDescription}

{------------------------------------------------------------------------------}
{ TdaTitanSQLADataSet.GetActive }

function TdaTitanSQLADataSet.GetActive: Boolean;
begin
  Result := GetQuery.Active
end; {function, GetActive}

{------------------------------------------------------------------------------}
{ TdaTitanSQLADataSet.SetActive }

procedure TdaTitanSQLADataSet.SetActive(Value: Boolean);
begin
  GetQuery.Active := Value;
end; {procedure, SetActive}

{------------------------------------------------------------------------------}
{ TdaTitanSQLADataSet.GetQuery }

function TdaTitanSQLADataSet.GetQuery: TtaQuery;
begin

  if (FQuery = nil) then
    FQuery := TtsQuery.Create(Self);

  Result := FQuery;

end; {procedure, GetQuery}

{------------------------------------------------------------------------------}
{ TdaTitanSQLADataSet.SetDatabaseName }

procedure TdaTitanSQLADataSet.SetDatabaseName(const aDatabaseName: String);
begin

  inherited SetDatabaseName(aDatabaseName);

  if GetQuery.Active then
    FQuery.Active := False;

  FQuery.DatabaseName := aDatabaseName;

end; {procedure, SetDatabaseName}

{------------------------------------------------------------------------------}
{ TdaTitanSQLADataSet.SetDataName }

procedure TdaTitanSQLADataSet.SetDataName(const aDataName: String);
begin

  inherited SetDataName(aDataName);

  {dataset cannot be active to set data name}
  if GetQuery.Active then
    FQuery.Active := False;

  {construct an SQL statment that returns an empty result set,
   this is used to get the field information }
  FQuery.SQL.Text := 'SELECT * FROM ' + aDataName +
                     ' WHERE ''c'' <> ''c'' ';

end; {procedure, SetDataName}

{------------------------------------------------------------------------------}
{ TdaTitanSQLADataSet.BuildFieldList }

procedure TdaTitanSQLADataSet.BuildFieldList;
var
  liIndex: Integer;
  lTitanField: TField;
  lField: TppField;
begin

  inherited BuildFieldList;

  if not(GetQuery.Active) then
    try
      FQuery.Active := True;
    except
      on E:Exception do
        Exit;
    end; {try, except}

  for liIndex := 0 to FQuery.FieldCount - 1 do
    begin
      lTitanField := FQuery.Fields[liIndex];

      lField := TppField.Create(nil);

      lField.TableName    := DataName;
      lField.FieldName    := lTitanField.FieldName;
      lField.DataType     := ppConvertFieldType(lTitanField.DataType);

      AddField(lField);
    end;

end; {function, BuildFieldList}

{------------------------------------------------------------------------------}
{ TdaTitanSQLADataSet.GetFieldNamesForSQL }

procedure TdaTitanSQLADataSet.GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings);
var
  lQuery: TtsQuery;
begin
  aList.Clear;

  lQuery := TtsQuery.Create(Self);

  lQuery.DatabaseName := DatabaseName;
  lQuery.SQL := aSQL;

  lQuery.GetFieldNames(aList);

  lQuery.Free;

end; {procedure, GetFieldNamesForSQL}

{------------------------------------------------------------------------------}
{ TdaTitanSQLADataSet.GetFieldsForSQL }

procedure TdaTitanSQLADataSet.GetFieldsForSQL(aList: TList; aSQL: TStrings);
var
  lQuery: TtsQuery;
  lTitanField: TField;
  lField: TppField;
  liIndex: Integer;
begin
  aList.Clear;

  lQuery := TtsQuery.Create(Self);

  lQuery.DatabaseName := DatabaseName;
  lQuery.SQL := aSQL;

  lQuery.Active := True;

  for liIndex := 0 to lQuery.FieldCount - 1 do
    begin
      lTitanField := lQuery.Fields[liIndex];

      lField := TppField.Create(nil);

      lField.FieldName    := lTitanField.FieldName;
      lField.DataType     := ppConvertFieldType(lTitanField.DataType);

      aList.Add(lField);
    end;

  lQuery.Free;

end; {procedure, GetFieldsForSQL}


{******************************************************************************
 *
 ** T I T A N   S Q L A   Q U E R Y   D A T A V I E W
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryDataView.Create }

constructor TdaTitanSQLAQueryDataView.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  {notes: 1. must use ChildQuery, ChildDataSource, ChildPipeline etc.
          2. use Self as owner for Query, DataSource etc.
          3. do NOT assign a Name }

  FQuery := TdaChildTitanSQLAQuery.Create(Self);
  FDataSource := TppChildDataSource.Create(Self);

  FDataSource.DataSet := FQuery;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryDataView.Destroy }

destructor TdaTitanSQLAQueryDataView.Destroy;
begin
  FDataSource.Free;
  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryDataView.SessionClass }

class function TdaTitanSQLAQueryDataView.SessionClass: TClass;
begin
  Result := TdaTitanSQLASession;
end; {class function, SessionClass}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryDataView.Loaded }

procedure TdaTitanSQLAQueryDataView.Loaded;
begin

  inherited Loaded;

  UpdateQueryObject;

end; {procedure, Loaded}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryDataView.ConnectPipelinesToData }

procedure TdaTitanSQLAQueryDataView.ConnectPipelinesToData;
begin

  if DataPipelineCount = 0 then Exit;

  {need to reconnect here}
  TppDBPipeline(DataPipelines[0]).DataSource := FDataSource;


end; {procedure, ConnectPipelinesToData}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryDataView.PreviewFormClass }

class function TdaTitanSQLAQueryDataView.PreviewFormClass: TFormClass;
begin
  Result := TFormClass(GetClass('TdaPreviewDataDialog'));
end; {class function, PreviewFormClass}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryDataView.Init }

procedure TdaTitanSQLAQueryDataView.Init;
var
  lDataPipeline: TppChildDBPipeline;

begin

  inherited Init;

  if DataPipelineCount > 0 then Exit;

  {note: DataView's owner must own the DataPipeline }
  lDataPipeline := TppChildDBPipeline(ppComponentCreate(Self, TppChildDBPipeline));
  lDataPipeline.DataSource := FDataSource;

  lDataPipeline.AutoCreateFields := False;

  {add DataPipeline to the dataview }
  lDataPipeline.DataView := Self;

end; {procedure, Init}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryDataView.SQLChanged }

procedure TdaTitanSQLAQueryDataView.SQLChanged;
begin
  UpdateQueryObject;
end; {procedure, SQLChanged}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryDataView.UpdateQueryObject }

procedure TdaTitanSQLAQueryDataView.UpdateQueryObject;
begin

  FQuery.DatabaseName := SQL.DatabaseName;
  FQuery.SQL := SQL.SQLText;

end; {procedure, UpdateQueryObject}



{******************************************************************************
 *
 ** T I T A N   S Q L A   T E M P L A T E  D A T A V I E W
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaTitanSQLATemplateDataView.IsTemplate }

class function TdaTitanSQLATemplateDataView.IsTemplate: Boolean;
begin
  Result := True;
end; {function, IsTemplate}

{------------------------------------------------------------------------------}
{ TdaTitanSQLATemplateDataView.SessionClass }

class function TdaTitanSQLATemplateDataView.SessionClass: TClass;
begin
  Result := TdaTitanSQLASession;
end; {class function, SessionClass}

{------------------------------------------------------------------------------}
{ TdaTitanSQLATemplateDataView.SetDatabaseName }

procedure TdaTitanSQLATemplateDataView.SetDatabaseName(aDatabaseName: String);
begin
  FDatabaseName := aDatabaseName;

end; {procedure, SetDatabaseName}


{------------------------------------------------------------------------------}
{ TdaTitanSQLATemplateDataView.CreateQuery }

function TdaTitanSQLATemplateDataView.CreateQuery: TtsQuery;
begin
  Result := TdaChildTitanSQLAQuery.Create(Self);
  Result.DatabaseName := DatabaseName;

end; {procedure, CreateQuery}

{------------------------------------------------------------------------------}
{ TdaTitanSQLATemplateDataView.CreateTable }

function TdaTitanSQLATemplateDataView.CreateTable(aTableName: String): TtsTable;
begin
  Result := TdaChildTitanSQLATable.Create(Self);
  Result.DatabaseName := DatabaseName;
  Result.TableName := aTableName;

end; {procedure, CreateTable}

{------------------------------------------------------------------------------}
{ TdaTitanSQLATemplateDataView.CreateStoredProc }

function TdaTitanSQLATemplateDataView.CreateStoredProc(aStoredProcName: String): TtsStoredProc;
begin
  Result := TdaChildTitanSQLAStoredProc.Create(Self);
  Result.DatabaseName := DatabaseName;
  Result.StoredProcName := aStoredProcName;

end; {procedure, CreateStoredProc}

{------------------------------------------------------------------------------}
{ TdaTitanSQLATemplateDataView.CreateDataSource }

function TdaTitanSQLATemplateDataView.CreateDataSource: TDataSource;
begin
  Result := TppChildDataSource.Create(Self);

end; {procedure, CreateDataSource}

{------------------------------------------------------------------------------}
{ TdaTitanSQLATemplateDataView.CreateDataPipeline }

function TdaTitanSQLATemplateDataView.CreateDataPipeline: TppDBPipeline;
begin

  {note: DataView's owner must own the DataPipeline }
  Result := TppChildDBPipeline(ppComponentCreate(Self, TppChildDBPipeline));

  {add pipeline to the dataview's Pipelines[] array}
  Result.DataView := Self;

end; {procedure, CreateDataPipeline}



{------------------------------------------------------------------------------}
{ TdaTitanSQLATemplateDataView.CreatePipelineField }

function TdaTitanSQLATemplateDataView.CreatePipelineField(aTableName, aFieldName, aFieldAlias: String;
                             aDataPipeline: TppDataPipeline; aSearchable, aSortable: Boolean): TppField;
begin

  Result := TppField.Create(nil);

  {set field props}
  Result.TableName     := aTableName;
  Result.FieldName     := aFieldName;

  if (aFieldAlias <> '') then
    Result.FieldAlias    := aFieldAlias
  else
    Result.FieldAlias    := aFieldName;

  Result.DataType      := aDataPipeline.GetFieldDataType(aFieldName);
  Result.FieldLength   := aDataPipeline.GetFieldSize(aFieldName);
  Result.DisplayWidth  := aDataPipeline.GetFieldDisplayWidth(aFieldName);

  Result.Searchable    := aSearchable;
  Result.Sortable      := aSortable;

  {add field to the datapipeline's Fields[] array}
  Result.DataPipeline := aDataPipeline;

  {turn auto create fields off}
  if (aDataPipeline <> nil) and (aDataPipeline is TppDBPipeline) and
    TppDBPIpeline(aDataPipeline).AutoCreateFields then
    TppDBPIpeline(aDataPipeline).AutoCreateFields := False;

end; {procedure, CreatePipelineField}

{******************************************************************************
 *
 ** Query Template DataView
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.Create }

constructor TdaTitanSQLAQueryTemplateDataView.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  {call CreateQuery method defined in the ancestor }
  FQuery := CreateQuery;

  FSQL := TdaSQL.Create(Self);
  FSQL.Session := TdaSessionClass(SessionClass).Create(Self);

  {call CreateDataSource method defined in the ancestor }
  FDataSource := CreateDataSource;
  FDataSource.DataSet := FQuery;

  FDataPipeline := nil;

  {AskAtRuntime := True; {This was commented out by default - JS}

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.Destroy }

destructor TdaTitanSQLAQueryTemplateDataView.Destroy;
begin

  inherited Destroy;

end; {destructor, Destroy}


{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.Loaded }


procedure TdaTitanSQLAQueryTemplateDataView.Loaded;
begin

  inherited Loaded;

  DefineSelectedFields;
  DefineDataSelection;

end; {procedure, Loaded}


{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.SetDatabaseName }

procedure TdaTitanSQLAQueryTemplateDataView.SetDatabaseName(aDatabaseName: String);
begin
  inherited SetDatabaseName(aDatabaseName);

  FQuery.DatabaseName := DatabaseName;
  FSQL.DatabaseName   := DatabaseName;

end; {procedure, SetDatabaseName}


{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.Init }


procedure TdaTitanSQLAQueryTemplateDataView.Init;
begin

  if (FDataPipeline <> nil) then Exit;

  DefineSelectedFields;
  DefineDataSelection;

  inherited Init;


end; {procedure, Init}



{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.CreateDataPipelines }

procedure TdaTitanSQLAQueryTemplateDataView.CreateDataPipelines;
begin

  {note: call CreateDataPipeline defined in ancestor }
  FDataPipeline := CreateDataPipeline;
  FDataPipeline.UserName := 'pl' + ppTextToIdentifier(ClassDescription);

end; {procedure, CreateDataPipelines}


{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.CreatePipelineFields }

procedure TdaTitanSQLAQueryTemplateDataView.CreatePipelineFields(aDataPipeline: TppDataPipeline);
var
  liIndex: Integer;

begin

  if (FDataPipeline = aDataPipeline) then
    begin

      TppDBPipeline(aDataPipeline).AutoCreateFields := False;

      for liIndex := 0 to FSQL.SelectFieldCount-1 do
        begin
          with FSQL.SelectFields[liIndex] do
            CreatePipelineField(TableAlias, FieldName, FieldAlias, aDataPipeline, Searchable, Sortable);

        end;

    end;

end; {procedure, CreatePipelineFields}


{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.ConnectPipelinesToData }

procedure TdaTitanSQLAQueryTemplateDataView.ConnectPipelinesToData;
begin

  if (FDatapipeline = nil) then Exit;

  {connect datapipeline to datasource here}
  FDatapipeline.DataSource := FDataSource;

end; {procedure, ConnectPipelinesToData}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.DefineSelectedFields }

procedure TdaTitanSQLAQueryTemplateDataView.DefineSelectedFields;
begin

  FSQL.Clear;

 {descendants add code here}

end; {procedure, DefineSelectedFields}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.DefineCalculatedFields }

procedure TdaTitanSQLAQueryTemplateDataView.DefineCalculatedFields;
begin

 {descendants add code here}

end; {procedure, DefineCalculatedFields}


{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.AddSelectTable }

function TdaTitanSQLAQueryTemplateDataView.AddSelectTable(aTableName: String): TdaTable;
begin
  Result := FSQL.AddTable(aTableName);

end; {function, AddSelectTable}

{------------------------------------------------------------------------------}
{ TdaTitanQueryTemplateDataView.AddSelectField }

function TdaTitanSQLAQueryTemplateDataView.AddSelectField(aTable: TdaTable; aFieldName, aFieldAlias: String; aSearchable, aSortable: Boolean): TdaField;
begin


  Result := FSQL.AddSelectField(aTable, aFieldName);

  if (aFieldAlias <> '')  then
    Result.FieldAlias := aFieldAlias;

  Result.Searchable := aSearchable;
  Result.Sortable   := aSortable;

end; {procedure, AddSelectField}


{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.DefineDataSelection }

procedure TdaTitanSQLAQueryTemplateDataView.DefineDataSelection;
begin

  SetActive(False);

  BuildSQL;

end; {procedure, DefineDataSelection}

{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.BuildSQL }

procedure TdaTitanSQLAQueryTemplateDataView.BuildSQL;
begin

  DefineSelectionCriteria;

  DefineSortOrder;

  FQuery.SQL := FSQL.SQLText;

end; {procedure, BuildSQL}


{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.DefineSelectionCriteria }

procedure TdaTitanSQLAQueryTemplateDataView.DefineSelectionCriteria;
var
  liIndex: Integer;
  lField: TppField;
  lTable: TdaTable;

begin

  if (FDataPipeline = nil) then Exit;


  {add code for selection criteria}
  FSQL.ClearCriteria;

  for liIndex := 0 to FDataPipeline.FieldCount-1 do
    if FDataPipeline.Fields[liIndex].Search then
      begin
        lField := FDataPipeline.Fields[liIndex];
        lTable := FSQL.GetTableForSQLAlias(lField.TableName);

        FSQL.AddCriteriaField(lTable, lField.FieldName, dacoLike, lField.SearchExpression);


      end;

end; {procedure, DefineSelectionCriteria}


{------------------------------------------------------------------------------}
{ TdaTitanSQLAQueryTemplateDataView.DefineSortOrder }

procedure TdaTitanSQLAQueryTemplateDataView.DefineSortOrder;

var
  liIndex: Integer;
  lField: TppField;
  ldaField: TdaField;
  lTable: TdaTable;
  lbAscending: Boolean;

begin

  if (FDataPipeline = nil) then Exit;

  {add code for sort order}
  FSQL.ClearOrderByFields;


  for liIndex := 0 to FDataPipeline.FieldCount-1 do
    if FDataPipeline.Fields[liIndex].Sort then
      begin

        lField := FDataPipeline.Fields[liIndex];

        lTable := FSQL.GetTableForSQLAlias(lField.TableName);

        ldaField := FSQL.CreateFieldForTable(lTable, lField.FieldName);

        lbAscending := (lField.SortType = soAscending);

        FSQL.AddOrderByField(ldaField, lbAscending);

      end;


end; {procedure, DefineSortOrder}

{******************************************************************************
 *
 ** I N I T I A L I Z A T I O N   /   F I N A L I Z A T I O N
 *
{******************************************************************************}

initialization

  RegisterClasses([TdaChildTitanSQLAQuery, TdaChildTitanSQLATable, TdaChildTitanSQLAStoredProc]);

  daRegisterSession(TdaTitanSQLASession);
  daRegisterDataSet(TdaTitanSQLADataSet);

  daRegisterDataView(TdaTitanSQLAQueryDataView);

finalization

  UnRegisterClasses([TdaChildTitanSQLAQuery, TdaChildTitanSQLATable, TdaChildTitanSQLAStoredProc]);

  daUnRegisterSession(TdaTitanSQLASession);
  daUnRegisterDataSet(TdaTitanSQLADataSet);

  daUnRegisterDataView(TdaTitanSQLAQueryDataView);

end.
